var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DeepClient, toPlain, useDeep } from '@deep-foundation/deeplinks';
import axios from 'axios';
import React, { createContext, useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react';
let _path, _ssl = true;
try {
    _path = process.env.GQL || process.env.NEXT_PUBLIC_GRAPHQL_URL || 'https://deeplinks.deep.foundation/gql';
}
catch (e) { }
try {
    _ssl = new URL(_path).protocol === "https:";
}
catch (e) { }
_path = _path.replace(/(^\w+:|^)\/\//, '');
const _secret = process.env.SECRET || process.env.DEEPLINKS_HASURA_SECRET;
const _token = process.env.TOKEN || process.env.NEXT_PUBLIC_DEEP_TOKEN;
export function toPlainPackages(packages) {
    const plain = [];
    for (let l = 0; l < packages.data.length; l++) {
        const link = packages.data[l];
        plain.push(toPlain(link));
        for (let v = 0; v < link._version.length; v++) {
            const version = link._version[v];
            plain.push(toPlain(version));
        }
    }
    return plain;
}
export function preloadQueries(deep) {
    return __awaiter(this, void 0, void 0, function* () {
        const Preload = yield deep.id('@deep-foundation/preload', 'Preload', true);
        const packagesQ = {
            type_id: {
                _nin: [
                    deep.idLocal('@deep-foundation/core', 'Promise'),
                    deep.idLocal('@deep-foundation/core', 'Then'),
                    deep.idLocal('@deep-foundation/core', 'Rejected'),
                    deep.idLocal('@deep-foundation/core', 'Resolved'),
                    deep.idLocal('@deep-foundation/core', 'PromiseResult'),
                    yield deep.id('@deep-foundation/tsx', 'TSX'),
                ]
            },
            up: {
                tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                parent: Object.assign(Object.assign({ type_id: { _eq: deep.idLocal('@deep-foundation/core', 'Package') } }, (Preload ? { in: { type_id: yield deep.id('@deep-foundation/preload', 'Preload') } } : {})), { string: { value: { _neq: 'deep' } } }),
            },
            return: {
                _version: {
                    relation: 'in',
                    type_id: deep.idLocal('@deep-foundation/core', 'PackageVersion')
                },
            },
        };
        const packagesO = { apply: 'packages' };
        const handlersQ = {
            execution_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'JSExecutionProvider') },
            return: {
                dist: { relation: 'dist' }
            },
        };
        const handlersO = { table: 'handlers' };
        return { packagesQ, packagesO, handlersQ, handlersO, };
    });
}
export function preloadApi(path = _path, secret = _secret, token = _token, ssl = _ssl) {
    return __awaiter(this, void 0, void 0, function* () {
        const p = path.replace(/(^\w+:|^)\/\//, '');
        const deep = new DeepClient({
            path: p,
            ssl: ssl,
            secret: secret,
            token: token,
            ws: true,
        });
        const preloaded = {
            packages: [],
            handlers: [],
        };
        const { packagesQ, packagesO, handlersQ, handlersO } = yield preloadQueries(deep);
        deep.subscribe(packagesQ, packagesO).subscribe({
            next: ({ plainLinks: packages }) => {
                preloaded.packages = packages;
            },
        });
        deep.subscribe(handlersQ, handlersO).subscribe({
            next: ({ data: handlers }) => {
                preloaded.handlers = handlers;
            },
        });
        let initial = true;
        return function handler(req, res) {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                if (initial) {
                    preloaded.packages = (_a = (yield deep.select(packagesQ, packagesO))) === null || _a === void 0 ? void 0 : _a.plainLinks;
                    preloaded.handlers = (_b = (yield deep.select(handlersQ, handlersO))) === null || _b === void 0 ? void 0 : _b.data;
                    initial = false;
                }
                console.log('preloadApi', { packages: preloaded.packages.length, handlers: preloaded.handlers.length });
                res.status(200).json(preloaded);
            });
        };
    });
}
export function getServerSidePropsPreload(arg, result) {
    return __awaiter(this, void 0, void 0, function* () {
        const preload = yield axios.get(`${process.env.__NEXT_PRIVATE_ORIGIN}/api/preload`);
        result.props = (result === null || result === void 0 ? void 0 : result.props) || {};
        result.props.preloaded = preload.data;
        console.log('getServerSidePropsPreload', { packages: preload.data.packages.length, handlers: preload.data.handlers.length });
        return result;
    });
}
export const PreloadContext = createContext(undefined);
export function usePreload() { return useContext(PreloadContext); }
export const HandlersContext = createContext([]);
export function useHandlersContext() { return useContext(HandlersContext); }
export function PreloadProviderCore({ preloaded = {}, children = null, }) {
    var _a, _b;
    const deep = useDeep();
    const [handlers, setHandlers] = useState((preloaded === null || preloaded === void 0 ? void 0 : preloaded.handlers) || []);
    useMemo(() => {
        deep.minilinks.add((preloaded === null || preloaded === void 0 ? void 0 : preloaded.packages) || [], 'preloader-packages');
    }, []);
    const [isPreloaded, setIsPreloaded] = useState(!!((_b = (_a = deep === null || deep === void 0 ? void 0 : deep.minilinks) === null || _a === void 0 ? void 0 : _a.links) === null || _b === void 0 ? void 0 : _b.length));
    const reload = useCallback(() => __awaiter(this, void 0, void 0, function* () {
        deep.minilinks.apply(yield deep.select({
            type_id: { _nin: [
                    deep.idLocal('@deep-foundation/core', 'Promise'),
                    deep.idLocal('@deep-foundation/core', 'Then'),
                    deep.idLocal('@deep-foundation/core', 'Rejected'),
                    deep.idLocal('@deep-foundation/core', 'Resolved'),
                    deep.idLocal('@deep-foundation/core', 'PromiseResult'),
                ] },
            up: {
                tree_id: { _eq: deep.idLocal('@deep-foundation/core', 'containTree') },
                parent: {
                    type_id: { _eq: deep.idLocal('@deep-foundation/core', 'Package') },
                    string: {}
                },
            },
        }), 'preloader');
    }), []);
    const _handlers = deep.useSubscription({
        execution_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'JSExecutionProvider') },
        return: {
            dist: { relation: 'dist' }
        },
    }, { table: 'handlers' });
    useEffect(() => {
        var _a;
        if (!!((_a = _handlers === null || _handlers === void 0 ? void 0 : _handlers.data) === null || _a === void 0 ? void 0 : _a.length))
            setHandlers(_handlers === null || _handlers === void 0 ? void 0 : _handlers.data);
    }, [_handlers]);
    const value = useMemo(() => ([isPreloaded, reload]), [isPreloaded]);
    const handlersRef = useRef(handlers);
    handlersRef.current = handlers;
    return React.createElement(PreloadContext.Provider, { value: value },
        React.createElement(HandlersContext.Provider, { value: handlersRef }, children));
}
export function PreloadProvider({ preloaded = {}, Editor, children = null, }) {
    const deep = useDeep();
    return React.createElement(React.Fragment, null,
        deep ? [React.createElement(PreloadProviderCore, { key: deep.linkId, preloaded: preloaded, children: children })] : children,
        ";");
    { }
}
//# sourceMappingURL=preload.js.map